class MsgMetaClass(type):
    def __new__(cls, name, bases, attr_dict):
        attr_dict["msg"] = {}
        for key, attr in attr_dict.items():
            if isinstance(attr, tuple):
                code, message = attr
                attr_dict[key] = code
                attr_dict["msg"][code] = message
        return super().__new__(cls, name, bases, attr_dict)



from flask_restful_resource.base_resource.base import BaseResource
from flask_restful_resource.base_resource.mongo_resource import MongoModelSchemaResource
from flask_restful_resource.base_resource.sql_resource import SQLModelSchemaResource



import json
import logging
import logging.handlers

import pytest
from flask import Flask, make_response
from flask_marshmallow import Marshmallow
from flask_mongoengine import MongoEngine
from flask_restful import Api
from flask_sqlalchemy import SQLAlchemy

from flask_restful_resource.base_resource.exceptions import ResourceException

CONSOLE_FORMAT = "%(asctime)s | %(levelname)s | %(lineno)s | %(funcName)s | %(message)s"
DATEFMT = "%Y-%m-%d %H:%M:%S"
console_handler = logging.StreamHandler()
console_handler.setLevel(logging.INFO)
console_handler.setFormatter(logging.Formatter(fmt=CONSOLE_FORMAT, datefmt=DATEFMT))
logging.basicConfig(level=logging.INFO, handlers=[console_handler])

app = Flask(__name__)
app.config["TESTING"] = True
app.config["SERVER_NAME"] = "127.0.0.1:5000"
api = Api(app)

# 初始化 mongoengine 配置
app.config.update({"MONGODB_SETTINGS": {"host": "mongodb://127.0.0.1:27017/xwjk_base"}})
mongodb = MongoEngine(app)

# 初始化 mysql 配置
app.config.update({"SQLALCHEMY_DATABASE_URI": "sqlite:///test.db", "SQLALCHEMY_TRACK_MODIFICATIONS": False})
sqldb = SQLAlchemy(app)
sqlma = Marshmallow(app)


@api.representation("application/json")
def output_json(data, code, headers):
    if isinstance(data, ResourceException):
        return data.get_response()

    resp = make_response(json.dumps({"code": 0, "msg": "OK", "data": data}), code)
    return resp


@pytest.fixture
def client():
    """
    返回 flask client
    """

    ctx = app.app_context()
    ctx.push()
    sqldb.create_all()
    yield app.test_client()
    mongodb.connection.drop_database("xwjk_base")
    sqldb.drop_all()
    ctx.pop()



import logging as logger

from marshmallow_mongoengine import ModelSchema

from flask_restful_resource.comm.utils import utc_timestamp

from .base import BaseResource
from .exceptions import ErrorCode, ResourceException

mongo_filter_op_map = {
    ">": "__gt",
    "<": "__lt",
    ">=": "__gte",
    "<=": "__lte",
    "!=": "__ne",
    "==": "",
    "contains": "contains",
}


class MongoModelResource(BaseResource):
    pk_name = "id"
    model = None
    list_schema = None
    detail_schema = None
    update_exclude_fields = []
    update_include_fields = []
    list_fields = []
    detail_fields = []
    filter_fields = []
    max_page_size = 100
    default_page_size = 10
    allow_query_all = False

    def get_queryset(self):
        filter_conditions = {}
        for filter_field in self.filter_fields:
            column, op, field, convert_fn = filter_field
            value = self.validate_data.get(field)
            if value is not None:
                value = convert_fn(value) if convert_fn else value
                operator = mongo_filter_op_map[op]
                filter_conditions["{column}{operator}".format(column=column, operator=operator)] = value

        queryset = self.model.objects.only(*self.list_fields).filter(**filter_conditions)
        return queryset

    def get(self):
        pk = self.validate_data.get(self.pk_name)
        if pk:
            try:
                instance = self.model.objects.only(*self.detail_fields).filter(pk=pk).first()
            except Exception as e:
                logger.warning(e)
                instance = None

            if not instance:
                return ResourceException(ErrorCode.INSTANCE_IS_NOT_EXIST)
            data, errors = self.detail_schema.dump(instance)
            if errors:
                logger.error(errors)
                return ErrorCode.SERIALIZATION_ERROR
            return data

        page = int(self.validate_data.get("page") or 1)
        page_size = min(int(self.validate_data.get("page_size") or self.default_page_size), self.max_page_size)

        queryset = self.get_queryset()
        if self.allow_query_all and page_size == -1:
            page_size = queryset.count()
        if page_size == 0:
            return {"total": 0, "pages": 0, "page": 1, "page_size": 0, "results": []}
        pagination = queryset.paginate(page=page, per_page=page_size)

        data, errors = self.list_schema.dump(pagination.items)
        if errors:
            logger.error(errors)
            return ErrorCode.SERIALIZATION_ERROR

        return {
            "total": pagination.total,
            "pages": pagination.pages,
            "page": pagination.page,
            "page_size": pagination.per_page,
            "results": data,
        }

    def post(self):
        instance, errors = self.detail_schema.load(self.validate_data)
        if errors:
            logger.info(str(errors))
            return ResourceException(ErrorCode.FIELD_TYPE_ERROR)
        instance.save()
        data, errors = self.detail_schema.dump(instance)
        if errors:
            logger.error(errors)
            return ErrorCode.SERIALIZATION_ERROR
        return data

    def pre_put_data(self, instance):
        return self.validate_data

    def put(self):
        instance = self.model.objects.filter(pk=self.validate_data.pop(self.pk_name)).first()
        if not instance:
            return ResourceException(ErrorCode.INSTANCE_IS_NOT_EXIST)
        exclude_fields = set(self.validate_data.keys()) & set(self.update_exclude_fields)
        include_fields = set(self.validate_data.keys()) - set(self.update_include_fields)
        if self.update_include_fields and include_fields:
            logger.info("Not Allowed To Update " + str(include_fields))
            return ResourceException(ErrorCode.NOT_ALLOW_UPDATE_FIELD_IS_EXIST)
        if exclude_fields:
            logger.info("Not Allowed To Update " + str(exclude_fields))
            return ResourceException(ErrorCode.NOT_ALLOW_UPDATE_FIELD_IS_EXIST)

        instance, errors = self.detail_schema.update(instance, self.pre_put_data(instance))
        if errors:
            logger.info(str(errors))
            return ResourceException(ErrorCode.FIELD_TYPE_ERROR)

        if hasattr(instance, "update_time"):
            instance.update_time = utc_timestamp()

        instance.save()
        data, errors = self.detail_schema.dump(instance)
        if errors:
            logger.error(errors)
            return ErrorCode.SERIALIZATION_ERROR
        return data

    def delete(self):
        pk = self.validate_data.get(self.pk_name)
        instance = self.model.objects.filter(pk=pk).first()
        if not instance:
            return ResourceException(ErrorCode.INSTANCE_IS_NOT_EXIST)
        instance.delete()


class MongoModelSchemaResource(MongoModelResource):
    def dispatch_request(self, *args, **kwargs):
        meta = type("Meta", (object,), dict(model=self.model))
        schema = type("Schema", (ModelSchema,), dict(Meta=meta))
        self.list_schema = schema(many=True)
        self.detail_schema = schema()
        return super().dispatch_request(*args, **kwargs)



import logging
import os

from flask_restful_resource.comm.zkCli import ZKClient


class EnvZk:
    """
    管理项目的参数配置，环境变量
    """

    def __init__(self, zk_servers, service_name, host):
        zk = ZKClient(zk_servers, service_name, host)
        zk.read_config()
        self._zk = zk
        self._conf_items = {}

    def init_conf(self, name, default=None, conftype=None, force=False, hash_log=None):
        """
        初始化参数配置
        """
        conf = ConfItem(self._zk, name, default, conftype, force, hash_log)
        self._conf_items[name] = conf
        return conf.value

    def __getitem__(self, key):
        """
        动态获取参数配置
        """
        try:
            return self._conf_items[key].value
        except KeyError:
            logging.error("getting conf failed. key [{}]".format(key))
            return None


class ConfItem:
    """
    保存每一项配置的附属信息
    """

    def __init__(self, zk, name, default=None, conftype=None, force=False, hash_log=None):
        self.zk = zk
        self.name = name
        self.default = default
        self.conftype = conftype
        self.force = force
        self.hash_log = hash_log

    @property
    def value(self):
        """
        读取配置变量

        读取顺序: 环境变量 > zk > default
        """

        env = os.environ.get(self.name)
        if not env:
            env = self.zk.config.get(self.name, None)
            if env is None or env == "":  # 防止 env=False, env=0 等现象
                env = self.default

        if (env is None or env == "") and self.force:
            raise Exception("Not found env: <{}>".format(self.name))

        if self.conftype == bool and not isinstance(env, bool):
            env = False if str(env) in ("0", "false", "False") else True
        elif self.conftype:
            env = self.conftype(env)

        if self.hash_log:
            logging.info("{}: {}".format(self.name, self.hash_log(env)))
        else:
            logging.info("{}: {}".format(self.name, env))

        return env



import logging as logger

from flask import current_app

from flask_restful_resource.comm.utils import utc_timestamp

from .base import BaseResource
from .exceptions import ErrorCode, ResourceException

filter_op_map = {
    ">": "__gt__",
    "<": "__lt__",
    ">=": "__ge__",
    "<=": "__le__",
    "!=": "__not__",
    "==": "__eq__",
    "contains": "contains",
}


class SQLModelResource(BaseResource):
    pk_name = "id"
    model = None
    args_schema = None
    list_schema = None
    detail_schema = None
    exclude = []
    list_fields = []
    update_exclude_fields = []
    update_include_fields = []
    filter_fields = []
    allow_query_all = False
    max_page_size = 100
    default_page_size = 10
    has_is_delete = False
    business_unique_fields = []

    @property
    def db(self):
        if "sqlalchemy" not in current_app.extensions:
            raise Exception("Not found [sqlalchemy] in [app.extensions]. Please init flask-sqlalchemy")
        return current_app.extensions["sqlalchemy"].db

    @property
    def marshmallow(self):
        if "flask-marshmallow" not in current_app.extensions:
            raise Exception("Not found [flask-marshmallow] in [app.extensions]. Please init flask-marshmallow")
        return current_app.extensions["flask-marshmallow"]

    def list_serializable(self, items):
        return self.list_schema.dump(items)

    def get_queryset(self):
        filter_conditions = []
        for filter_field in self.filter_fields:
            column, op, field, convert_fn = filter_field
            value = self.validate_data.get(field)
            if value is not None:
                value = convert_fn(value)
                column = getattr(self.model, column)
                operator = filter_op_map[op]
                filter_conditions.append(getattr(column, operator)(value))

        queryset = self.model.query.filter(*filter_conditions)
        if self.has_is_delete:
            queryset = queryset.filter_by(is_delete=False)
        return queryset

    def get(self):
        pk = self.validate_data.get(self.pk_name)
        if pk:
            instance = self.model.query.get(pk)
            if not instance:
                return ResourceException(ErrorCode.INSTANCE_IS_NOT_EXIST)
            data, errors = self.detail_schema.dump(instance)
            if errors:
                logger.error(errors)
                return ErrorCode.SERIALIZATION_ERROR
            return data

        page = int(self.validate_data.get("page") or 1)
        page_size = min(int(self.validate_data.get("page_size") or self.default_page_size), self.max_page_size)

        queryset = self.get_queryset()
        if self.allow_query_all and page_size == -1:
            page_size = queryset.count()

        pagination = queryset.paginate(page=page, per_page=page_size)
        results, errors = self.list_serializable(pagination.items)
        if errors:
            logger.error(errors)
            return ErrorCode.SERIALIZATION_ERROR

        if self.list_fields:
            results = [{k: v for k, v in data.items() if k in self.list_fields} for data in results]

        return {
            "total": pagination.total,
            "pages": pagination.pages,
            "page": pagination.page,
            "page_size": pagination.per_page,
            "results": results,
        }

    def post(self):
        filter_conditions = []
        if self.business_unique_fields:
            basequery = self.model.query
            if self.has_is_delete:
                basequery = basequery.filter_by(is_delete=False)
            for field in self.business_unique_fields:
                column = getattr(self.model, field)
                filter_conditions.append(getattr(column, "__eq__")(self.validate_data.get(field)))
            instance = basequery.filter(*filter_conditions).first()
            if instance:
                logger.info("instance is exist")
                return ResourceException(ErrorCode.INSTANCE_IS_EXIST)

        instance, errors = self.detail_schema.load(self.validate_data)
        if errors:
            logger.info(errors)
            return ResourceException(ErrorCode.FIELD_TYPE_ERROR)
        try:
            self.db.session.add(instance)
            self.db.session.commit()
            data, errors = self.detail_schema.dump(instance)
            if errors:
                logger.error(errors)
                return ErrorCode.SERIALIZATION_ERROR
            return data

        except Exception as e:
            logger.info(str(e))
            return ResourceException(ErrorCode.DB_COMMIT_ERROR)

    def pre_put_data(self, instance):
        return self.validate_data

    def put(self):
        instance = self.model.query.get(self.validate_data.get(self.pk_name))
        if not instance:
            return ResourceException(ErrorCode.INSTANCE_IS_NOT_EXIST)
        exclude_fields = set(self.validate_data.keys()) & set(self.update_exclude_fields)
        include_fields = set(self.validate_data.keys()) - set(self.update_include_fields) - set([self.pk_name])
        if self.update_include_fields and include_fields:
            logger.info("Not allowed to update " + str(include_fields))
            return ResourceException(ErrorCode.NOT_ALLOW_UPDATE_FIELD_IS_EXIST)
        if exclude_fields:
            logger.info("Not allowed to update " + str(exclude_fields))
            return ResourceException(ErrorCode.NOT_ALLOW_UPDATE_FIELD_IS_EXIST)

        instance, errors = self.detail_schema.load(self.pre_put_data(instance), partial=True)
        if errors:
            logger.info(str(errors))
            return ResourceException(ErrorCode.FIELD_TYPE_ERROR)

        if hasattr(instance, "update_time"):
            instance.update_time = utc_timestamp()

        self.db.session.commit()
        data, errors = self.detail_schema.dump(instance)
        if errors:
            logger.error(errors)
            return ErrorCode.SERIALIZATION_ERROR
        return data

    def delete(self):
        pk = self.validate_data.get(self.pk_name)
        instance = self.model.query.get(pk)
        if not instance:
            return ResourceException(ErrorCode.INSTANCE_IS_NOT_EXIST)
        if self.has_is_delete:
            instance.is_delete = True
        else:
            self.db.session.delete(instance)
        self.db.session.commit()


class SQLModelSchemaResource(SQLModelResource):
    def dispatch_request(self, *args, **kwargs):
        meta = type("Meta", (object,), dict(model=self.model))
        schema = type("Schema", (self.marshmallow.ModelSchema,), dict(Meta=meta))
        self.list_schema = schema(many=True, exclude=self.exclude)
        self.detail_schema = schema(exclude=self.exclude)
        return super().dispatch_request(*args, **kwargs)



import logging as logger

from flask import request
from flask_restful import Resource, reqparse
from schema import Schema

from flask_restful_resource.comm.utils import move_space, validate_schema

from .exceptions import ErrorCode, ResourceException


class BaseResource(Resource):
    validate_data = None
    validate_schemas = {}
    allow_methods = ["get", "post", "put", "delete"]

    def dispatch_request(self, *args, **kwargs):
        if request.method.lower() not in self.allow_methods:
            return ResourceException(ErrorCode.METHOD_NOT_ALLOW, 405)
        req = None
        schema = None
        if request.method == "GET":
            schema = self.validate_schemas.get("get")
            req = request.args.to_dict()
        else:
            schema = self.validate_schemas.get(request.method.lower())
            req = request.json
        req = move_space(req)
        if isinstance(schema, Schema):
            data, errors = validate_schema(schema, req)
            if errors:
                logger.info(str(errors))
                return ResourceException(ErrorCode.VALIDATE_ERROR)
            self.validate_data = data
        elif isinstance(schema, reqparse.RequestParser):
            parse_req = schema.parse_args()
            strict = self.validate_schemas.get("strict") or ""
            if strict:
                parse_req = schema.parse_args(strict=True)
            self.validate_data = {k: v for k, v in parse_req.items() if v}

        if not self.validate_data:
            self.validate_data = req
        return super().dispatch_request(*args, **kwargs)

    def get(self):
        pass

    def put(self):
        pass

    def delete(self):
        pass



import pytest
from flask import url_for
from schema import Optional, Schema, Use

from flask_restful_resource import MongoModelSchemaResource
from flask_restful_resource.base_resource.exceptions import ErrorCode

from .conftest import api
from .conftest import mongodb as db


class Person(db.Document):
    meta = {"collection": "person", "indexes": ["name", "age"]}

    name = db.StringField(required=True)
    age = db.IntField(required=True)


class MyMongo(MongoModelSchemaResource):

    model = Person
    allow_query_all = True
    # allow_methods = ["get", "post", "put", "delete"]

    filter_fields = [["name", "==", "name", str], ["age", "==", "age", None]]

    list_fields = ["name", "age"]

    update_include_fields = ["name", "age"]

    validate_schemas = {
        "get": Schema(
            {Optional("id"): str, Optional("name"): str, Optional("age"): Use(int), Optional("page_size"): Use(int)}
        ),
        "post": Schema({"name": str, "age": Use(int), Optional("page_size"): Use(int)}),
        "put": Schema({"id": str, Optional("name"): str, "age": Use(int), Optional("page_size"): Use(int)}),
        "delete": Schema({"id": str}),
    }

    def pre_put_data(self, instance):
        self.validate_data["name"] = instance.name + ":" + self.validate_data["name"]
        return self.validate_data


api.add_resource(MyMongo, "/mongo")


@pytest.mark.parametrize("_id", ["123", "123123123123123123123123"])
def test_get_detail_none(client, _id):
    result = client.get(url_for("mymongo"), query_string={"id": _id}).json
    assert result["code"] == ErrorCode.INSTANCE_IS_NOT_EXIST


def test_get_success(client):
    result = client.get(url_for("mymongo"), query_string={"name": "jack", "age": 18, "page_size": -1}).json
    assert result["data"] == {"total": 0, "pages": 0, "page": 1, "page_size": 0, "results": []}


def test_post_success(client):
    result = client.post(url_for("mymongo"), json={"name": "jack", "age": 18}).json
    assert result["code"] == 0


def test_post_fail(client):
    result = client.post(url_for("mymongo"), query_string={"name": "jack"}).json
    assert result["code"] == ErrorCode.VALIDATE_ERROR


def test_put_success(client):
    result = client.post(url_for("mymongo"), json={"name": "jack", "age": 18}).json
    _id = result["data"]["id"]
    result = client.put(url_for("mymongo"), json={"id": _id, "name": "jack1", "age": 19}).json
    assert result["data"]["age"] == 19
    assert Person.objects.first().age == 19


def test_put_fail(client):
    result = client.post(url_for("mymongo"), json={"name": "jack", "age": 18}).json
    _id = "123123123123123123123123"
    result = client.put(url_for("mymongo"), json={"id": _id, "name": "jack1", "age": 19}).json
    assert result["code"] == ErrorCode.INSTANCE_IS_NOT_EXIST


def test_delete_success(client):
    result = client.post(url_for("mymongo"), json={"name": "jack", "age": 18}).json
    _id = result["data"]["id"]
    result = client.delete(url_for("mymongo"), json={"id": _id}).json
    assert result["code"] == 0

    result = client.get(url_for("mymongo")).json
    assert result["data"]["total"] == 0


def test_pre_put_data(client):
    result = client.post(url_for("mymongo"), json={"name": "jack", "age": 18}).json
    _id = result["data"]["id"]
    result = client.put(url_for("mymongo"), json={"id": _id, "name": "jack1", "age": 19}).json
    assert result["data"]["age"] == 19
    assert result["data"]["name"] == "jack:jack1"



import logging
import time
from datetime import datetime

from schema import Schema, SchemaError


def move_space(data: dict):
    if data:
        for k, v in data.items():
            if isinstance(v, str):
                data[k] = str.strip(v)
        return data
    return {}


def validate_schema(schema: Schema, data: dict, remove_blank=False):
    """schema验证,验证成功返回数据，验证失败返回错误信息
    Parameters
    ----------
    schema:Schema: 验证规则
    data: 验证数据
    remove_blank : 是否去除空白字段

    Returns (data,errors)
    -------

    """
    if not isinstance(data, dict):
        return {}, "Not found params"
    d = {}
    if remove_blank:
        for k, v in data.items():
            if v != "":
                d[k] = v
    else:
        d = data
    try:
        validate_data = schema.validate(d)
        return validate_data, []
    except SchemaError as e:
        return {}, str(e.autos)
    else:
        return validate_data, []


def utc_timestamp():
    """返回utc时间戳（秒）"""
    return int(datetime.now().timestamp())


def utc_strftime(fmt):
    return datetime.now().strftime(fmt)


def print_run_time(func):
    """计算时间函数"""

    def wrapper(*args, **kw):
        local_time = int(time.time() * 1000)
        result = func(*args, **kw)
        logging.info("current Function [%s] run time is %s ms" % (func.__name__, int(time.time() * 1000) - local_time))
        return result

    return wrapper



from flask import url_for
from schema import Schema, Use

from flask_restful_resource import BaseResource
from flask_restful_resource.base_resource.exceptions import ErrorCode

from .conftest import api


class MyBase(BaseResource):

    validate_schemas = {"get": Schema({"name": str, "age": Use(int)})}

    allow_methods = ["get"]

    def get(self):
        return self.validate_data


api.add_resource(MyBase, "/base")


def test_get_success(client):
    result = client.get(url_for("mybase"), query_string={"name": "jack", "age": 18}).json
    assert result["data"] == {"name": "jack", "age": 18}


def test_get_fail(client):
    result = client.get(url_for("mybase"), query_string={"name": "jack"}).json
    assert result["code"] == ErrorCode.VALIDATE_ERROR



import json

from flask import make_response

from flask_restful_resource.comm.config import MsgMetaClass


class ResourceException:
    def __init__(self, code, status_code=200):
        self.code = code
        self.status_code = status_code
        self.msg = ErrorCode.msg[code]

    def get_response(self, code_map=None):

        if code_map and isinstance(code_map, dict):
            self.code = code_map.get(self.code, self.code)

        return make_response(json.dumps({"code": self.code, "msg": self.msg, "data": None}), self.status_code)


class ErrorCode(metaclass=MsgMetaClass):
    METHOD_NOT_ALLOW = (9999000001, "不支持该请求")
    VALIDATE_ERROR = (9999000002, "字段未通过验证")
    FIELD_TYPE_ERROR = (9999000003, "字段类型错误")
    INSTANCE_IS_NOT_EXIST = (9999000004, "不存在该记录")
    INSTANCE_IS_EXIST = (9999000005, "该记录已经存在")
    NOT_ALLOW_UPDATE_FIELD_IS_EXIST = (9999000006, "存在不允许更新的字段")
    SERIALIZATION_ERROR = (9999000007, "序列化出错")
    DB_COMMIT_ERROR = (9999000006, "数据库错误")



#!usr/bin/env python
# coding=utf-8
"""
@author: holens
@time: 2018/09/02
@desc:
"""
import json
import logging

from kazoo.client import KazooClient


class ZKClient:
    def __init__(self, zk_servers, service_name, host):
        self.config = None
        self.zk = KazooClient(zk_servers)
        self.zk.start()

        self.service_name = service_name
        self.serve_path = "/entry/service/{}/node".format(service_name)
        self.zk.ensure_path(self.serve_path)
        self.zk.create(self.serve_path + "/server", host.encode(), ephemeral=True, sequence=True)
        self.config_path = "/entry/config/service/{}".format(self.service_name)
        self.zk.DataWatch(self.config_path, self.read_config)

    def read_config(self, *args):
        logging.info("ZKClient: Parameter configuration updated")
        self.zk.ensure_path("/entry/config/service")
        if not self.zk.exists(self.config_path):
            self.zk.create(self.config_path, json.dumps({}).encode())
        self.config = json.loads(self.zk.get(self.config_path)[0].decode())

    def update_config(self, config):
        self.zk.set(self.config_path, json.dumps(config).encode())



import logging as logger
import time

import requests


def get_host_ip(reflect_service, service_name):
    if not reflect_service:
        raise Exception("REFLECT_SERVICE is None.")

    logger.info("Getting service registration address...")

    url = "{}?service={}".format(reflect_service, service_name)
    response = requests.get(url, timeout=10)
    if not response.status_code == 200:
        raise Exception("Request incorrect: <{}>".format(url))

    data = response.json()
    host, port = data["host"], data["port"]
    if host:
        host_ip = "{}:{}".format(host, port)
        logger.info("Getting success <{}>".format(host_ip))
        return host_ip

    logger.info("Getting failed <{}>, Retry after 3 second...".format(url))
    time.sleep(3)
    return get_host_ip(reflect_service, service_name)



from setuptools import find_packages, setup

setup(
    name="flask_restful_resource",
    version="0.3.1",
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "Flask==1.0.2",
        "Flask-RESTful==0.3.6",
        "schema==0.6.8",
        "marshmallow==2.16.3",
        "requests==2.19.1",
        "kazoo==2.5.0",
        # mongo
        # "marshmallow-mongoengine==0.9.1",
        # "flask-mongoengine==0.9.5",
        # sql
        # "Flask-SQLAlchemy==2.3.2",
        # "flask-marshmallow==0.9.0",
        # "marshmallow-sqlalchemy==0.15.0",
    ],
)



import time

import pytest

from flask_restful_resource.comm.env import EnvZk


@pytest.mark.skip(reason="skip")
def test_one(client):
    ez = EnvZk("111.230.231.89:2181,111.230.231.89:2182,111.230.231.89:2183", "chaos", "localhost:12345")

    ez.init_conf("CUR_ENV", force=True)
    ez.init_conf("CHAOS_DB", force=True)
    ez.init_conf("SENTRY_URL")
    ez.init_conf("REDIS_URL", force=True)
    ez.init_conf("LOG_DB", force=True)
    ez.init_conf("SQLALCHEMY_TRACK_MODIFICATIONS", default=False, conftype=bool)
    ez.init_conf("SQLALCHEMY_POOL_SIZE", default=5, conftype=int)
    ez.init_conf("ALERT_SENTRY_DNS")
    ez.init_conf("SMS_SENDING_LIMIT", default=100, conftype=int)
    ez.init_conf("VERIFY_CODE_EXPIRE", default=2, conftype=int)
    ez.init_conf("VERIFY_CODE_EXPIRE_MAIL", default=5, conftype=int)
    ez.init_conf("AMAP_SERVICE_KEY", force=True)
    ez.init_conf("IPV4_DATX_URL", force=True)
    ez.init_conf("MONGO_URI", default="ssssss", force=True)
    ez.init_conf("IS_SEND_SMS", default=True, conftype=bool)
    ez.init_conf("IS_SKIP_CFCA", default=False, conftype=bool)
    ez.init_conf("REPORT_EXC_URL")
    for _ in range(1000):
        time.sleep(1)
        ez["REPORT_EXC_URL"]
    assert 1



#!usr/bin/env python
# coding=utf-8
"""
@author: holens
@time: 2018/09/02
@desc: 线程安全的logger
"""
import logging
import logging.handlers

logger_dict = {}

CONSOLE_FORMAT = "%(asctime)s | %(levelname)s | %(lineno)s | %(funcName)s | %(message)s"
DATEFMT = "%Y-%m-%d %H:%M:%S"


def get_logger(name, level=logging.INFO, fmt=CONSOLE_FORMAT, datefmt=DATEFMT):
    """
    返回一个logger对象
    :param name:
    :return:logger对象
    """
    # 判断logger对象是否存在
    if name in logger_dict:
        return logger_dict[name]

    logger = logging.getLogger(name)

    logger.setLevel(level)
    # console handler
    console_handler = logging.StreamHandler()
    console_handler.setLevel(level)
    console_handler.setFormatter(logging.Formatter(fmt=fmt, datefmt=datefmt))

    logger.addHandler(console_handler)

    # 字典保存logger对象
    logger_dict[name] = logger

    logging.basicConfig(level=level, handlers=[console_handler])
    logger.propagate = False
    return logger
